#%%
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib import gridspec
import os
import csv

plt.rcParams['xtick.direction'] = 'in'
plt.rcParams['ytick.direction'] = 'in'
plt.rcParams['font.size'] = 12
plt.rcParams['text.usetex'] = True
plt.rcParams['text.color'] = 'black'
plt.rcParams['axes.spines.top'] = False
plt.rcParams['axes.spines.right'] = False

# Functions / Definitions
def cm2inch(*tupl):
    inch = 2.54
    if isinstance(tupl[0], tuple):
        return tuple(i/inch for i in tupl[0])
    else:
        return tuple(i/inch for i in tupl)

    
# Read the specified file and return the second and third columns
def read_file(file_path):
    with open(file_path, 'r') as file:
        data = np.genfromtxt(file, skip_header=1)
        all_data.append((data[1],data[-2]))

def cantilever_deflection(q):
    # Deflection according to beam theory
    b = 1.
    h = 0.1
    E = 2e6
    I = b*h**3/12
    L = 1
    return q * L**4 / (8*E*I)

def cantilever_deformation2(q, x):
    # Parameters for the beam
    b = 1.
    h = 0.1
    L = 1.
    E = 2e6
    I = b * h**3 / 12
    # Deflection as a function of position x
    y = (q*L**4) / (24*E*I) * ((x/L)**4 -4*(x/L)**3 + 6*(x/L)**2)
    return y

fig_width = 10.
fig_height = 9.

elements = ['u4','u3','u6','u8','u8-red']
titles = ['u4-solid','u3-solid','u6-solid','u8-solid','u8-solid-red']
distances = ['5e-2','25e-3','1e-2']
labels = ['$\Delta x = 5 \cdot 10^{-2}$ m', '$\Delta x = 2.5 \cdot 10^{-2}$ m', '$\Delta x = 1 \cdot 10^{-2}$ m']
linestyles = [':', '-.', '-']

q = np.linspace(1,20,20)
w = cantilever_deflection(q)

x_values = np.arange(0.,1.1,0.1)
y_values = np.array([cantilever_deformation2(20,x) for x in x_values])

# Begin figure
for ielem, ititle in zip(elements,titles):

    fig = plt.figure(figsize=cm2inch(2.*fig_width, fig_height))
    plt.title(r'Element type: ' + str(ititle), fontsize=12)

    for idist, ils, ilabel in zip(distances, linestyles, labels):

        all_data = []
        directory='./'+ielem+'/'+ielem+'_loclen_'+idist+'-print-out/'

        # Iterate over all files in the directory
        for file_name in os.listdir(directory):
            if file_name.endswith('.dat'):  # Ensure it's a CSV file
                file_path = os.path.join(directory, file_name)
                read_file(file_path)

        # Sort data based on the position (distance)
        sorted_data = np.array(sorted(all_data, key=lambda x: x[0]))
        plt.plot(sorted_data[:,0], sorted_data[:,1], 'b', ls=ils, label=ilabel)

    plt.xlim(-0.05,1.05)
    plt.xlabel('Length along beam in m')
    plt.ylim(-0.02,0.001)
    plt.ylabel('Deflection in m')

    plt.plot(x_values, -y_values, 'k', ls='', marker='o', markersize=6, markerfacecolor='none', label='Analytical')
    plt.legend(ncol=1, frameon=False, loc='lower left', bbox_to_anchor=(0.0, 0.0))

    plt.tight_layout()

    fig.savefig('cantilever_'+ielem+'.pdf', bbox_inches='tight', dpi=400)
    fig.savefig('cantilever_'+ielem+'.svg', bbox_inches='tight', dpi=400)
    fig.savefig('cantilever_'+ielem+'.png', bbox_inches='tight', dpi=400)

    plt.show()
    plt.close()
